//
//  CalendarViewController.swift
//  CalendarTest
//
//  Created by Don Rathwell May 26th 2016
//  Copyright © 2016 Don Rathwell. All rights reserved.
//
//
//

import UIKit
import JTAppleCalendar
import EventKit


class CalendarViewController: UIViewController {
    @IBOutlet weak var monthLabel: UILabel!
    @IBOutlet weak var calendarView: JTAppleCalendarView!
   
    
    
    
    let formatter = NSDateFormatter()
    var testCalendar: NSCalendar! = NSCalendar(calendarIdentifier: NSCalendarIdentifierGregorian)

    // to hold event information
    var dayEvents : [EKEvent] = []
    var startDates : [NSDate] = []

    //to pass to other classes
    var passImage = UIImage()
    var badWolfImage = UIImage()
    var goodWolfImage = UIImage()
    let wolfHeadSize = CGSizeMake(30,30)
    var passCellDate = NSDate()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        calendarView.delegate = self
        calendarView.dataSource = self
        //print(calendarView)
        //print(calendarView.dataSource)
        calendarView.registerCellViewXib(fileName: "CellViewer")
        let tempDate = NSDate()
        calendarView.scrollToDate(tempDate, triggerScrollToDateDelegate: true)
        getEventInfo()
        
        
        //Something goofy is happening with this
        // Enable the following code line to show headers. There are other lines of code to uncomment as well
        //calendarView.registerHeaderViewXibs(fileNames: ["PinkSectionHeaderView", "WhiteSectionHeaderView"]) // headers are Optional. You can register multiple if you want.
        
        
        
        
        
    
        
        //First attempt fix layout
       /* view.translatesAutoresizingMaskIntoConstraints = false
        // Pin view to all four edges of its superview
        calendarView.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("H:|[view]|", options: [], metrics: nil, views: ["view": view]))
        calendarView.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|[view]|", options: [], metrics: nil, views: ["view": view]))*/
       
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    //to get event information
    func getEventInfo(){
        
        let eventStore = EKEventStore()
        let calendars = eventStore.calendarsForEntityType(.Event)
        getWolfHeads()
        for calendar in calendars {
            if calendar.title == "HabitName" {
                
                let oneMonthAgo = NSDate(timeIntervalSinceNow: -30*24*3600)
                let oneMonthAfter = NSDate(timeIntervalSinceNow: +30*24*3600)
                let predicate = eventStore.predicateForEventsWithStartDate(oneMonthAgo, endDate: oneMonthAfter, calendars: [calendar])
                let events = eventStore.eventsMatchingPredicate(predicate)
                for event in events {
                    dayEvents.append(event)
                    //print(event.title)
                    //print(event.startDate)
                    //print(event.notes)
                }
            }
        }
    }
    
    //match event to calendar
    func getStartDates() -> [NSDate] {
        return startDates
    }
    
    
    
    

    
    func getCalendar () -> NSCalendar{
        let tempCal =  NSCalendar.currentCalendar()
        return tempCal
    }
    
    func getCalID () -> String{
        
        let eventStore = EKEventStore()
        let calendars = eventStore.calendarsForEntityType(EKEntityType.Event) as [EKCalendar]
        var temp = ""
        
        for calendar in calendars {
            if calendar.title == "HabitName" {
                temp = calendar.calendarIdentifier
            }
        }
        return String(temp)
    }
    
    @IBAction func printSelectedDates() {
        print("Selected dates --->")
        for date in calendarView.selectedDates {
            print(formatter.stringFromDate(date))
        }
    }
    
    func setupViewsOfCalendar(startDate: NSDate, endDate: NSDate) {
        let month = testCalendar.component(NSCalendarUnit.Month, fromDate: startDate)
        let monthName = NSDateFormatter().monthSymbols[(month-1) % 12] // 0 indexed array
        let year = NSCalendar.currentCalendar().component(NSCalendarUnit.Year, fromDate: startDate)
        monthLabel.text = monthName + " " + String(year)
    }
    
    func printSelectedDates(cellState: CellState) {
        //print("Selected dates --->")
        //print(cellState.text)
        for date in calendarView.selectedDates {
            print(formatter.stringFromDate(date))
        }
    }
    
 
    
    func getEventsForThisDay() -> [EKEvent]{
        var successCount = 0
        var failureCount = 0
        var passEvents : [EKEvent] = []
        
        for event in dayEvents {
            if testCalendar.isDate(event.startDate, inSameDayAsDate : passCellDate){
                if event.title == "success"{
                    successCount += 1
                    
                }else if event.title == "failure"{
                    failureCount += 1
                }
                
                passEvents.append(event)
                
                if failureCount < successCount{
                    if let tempImage = UIImage(named:"goodWolf3"){
                        passImage = imageResize(tempImage, sizeChange: wolfHeadSize)
                        
                    }
                    }else  if let tempImage = UIImage(named:"badWolf2"){
                        passImage = imageResize(tempImage, sizeChange: wolfHeadSize)
                    }
                }
            }
            return passEvents
    }
    func imageResize (image: UIImage, sizeChange:CGSize)-> UIImage{
        
        let hasAlpha = true
        let scale: CGFloat = 0.0 // Use scale factor of main screen
        UIGraphicsBeginImageContextWithOptions(sizeChange, !hasAlpha, scale)
        image.drawInRect(CGRect(origin: CGPointZero, size: sizeChange))
        let scaledImage = UIGraphicsGetImageFromCurrentImageContext()
        return scaledImage
    }
    func getWolfHeads(){
        
        
        if let tempImage = UIImage(named:"goodWolf3"){
            goodWolfImage = imageResize(tempImage, sizeChange: wolfHeadSize)
        }
        if let tempImage2 = UIImage(named:"badWolf2"){
            badWolfImage = imageResize(tempImage2, sizeChange: wolfHeadSize)
            
            
        }
    }
    
    // MARK: Navigation
    
    
    
    // This method lets you configure a view controller before it's presented.
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        if segue.identifier == "Show Detail"{
        
            let detailViewController = segue.destinationViewController as! DetailViewController
            let displayEvents = getEventsForThisDay()
          
            //pass stuff over
            detailViewController.dayEvents = displayEvents
            //print(displayEvents.count)
            detailViewController.goodWolfImage = goodWolfImage
            detailViewController.badWolfImage = badWolfImage
           
        }
    }
    
    
    @IBAction func prepareForUnwind(sender: UIStoryboardSegue) {
    }
    

    
}

extension CalendarViewController: JTAppleCalendarViewDataSource, JTAppleCalendarViewDelegate  {
   
    // Setting up manditory protocol method
    func configureCalendar(calendar: JTAppleCalendarView) -> (startDate: NSDate, endDate: NSDate, numberOfRows: Int,   calendar: NSCalendar) {
        
        // You can set your date using NSDate() or NSDateFormatter. Your choice.
        let formatter = NSDateFormatter()
        formatter.dateFormat = "yyyy MM dd"
        
        let firstDate = formatter.dateFromString("2016 01 05")
        //let firstDate = NSDate()
        let secondDate = formatter.dateFromString("2017 01 05")
        let numberOfRows = 6
        let aCalendar = getCalendar() // Properly configure your calendar to your time zone here
        return (startDate: firstDate!, endDate: secondDate!, numberOfRows: numberOfRows, calendar: aCalendar)
    }
    
    
    func calendar(calendar: JTAppleCalendarView, isAboutToDisplayCell cell: JTAppleDayCellView, date: NSDate, cellState: CellState) {
        
        (cell as! CellViewer).setupCellBeforeDisplay(cellState, date: date)
        
    }
    func calendar(calendar: JTAppleCalendarView, didDeselectDate date: NSDate, cell: JTAppleDayCellView?, cellState: CellState) {
        (cell as? CellViewer)?.cellSelectionChanged(cellState)
        (cell as? CellViewer)?.dateLabel.backgroundColor = UIColor.whiteColor()//********************************************************************
        // This is where I'm changing the color to confirm interaction
        //print("deselect " + cellState.text)
        
    }
    
    func calendar(calendar: JTAppleCalendarView, didSelectDate date: NSDate, cell: JTAppleDayCellView?, cellState: CellState) {
        (cell as? CellViewer)?.cellSelectionChanged(cellState)
        (cell as? CellViewer)?.dateLabel.backgroundColor = UIColor.redColor() //*********************************************************
        // This is where I'm changing the color to confirm interaction
        
        //load detail view
        
        passCellDate = cellState.date
        
        self.performSegueWithIdentifier("Show Detail", sender: self)
        
        
        //printSelectedDates(cellState)
    }
    
    
    func calendar(calendar: JTAppleCalendarView, didScrollToDateSegmentStartingWithdate startDate: NSDate, endingWithDate endDate: NSDate) {
        setupViewsOfCalendar(startDate, endDate: endDate)
    }
    
    func calendar(calendar: JTAppleCalendarView, sectionHeaderIdentifierForDate date: (startDate: NSDate, endDate: NSDate)) -> String? {
        let comp = testCalendar.component(.Month, fromDate: date.startDate)
        if comp % 2 > 0{
            return "WhiteSectionHeaderView"
        }
        return "PinkSectionHeaderView"
    }
    
    func calendar(calendar: JTAppleCalendarView, sectionHeaderSizeForDate date: (startDate: NSDate, endDate: NSDate)) -> CGSize {
        if testCalendar.component(.Month, fromDate: date.startDate) % 2 == 1 {
            return CGSize(width: 200, height: 50)
        } else {
            return CGSize(width: 200, height: 100) // Yes you can have different size headers
        }
    }
    
    func calendar(calendar: JTAppleCalendarView, isAboutToDisplaySectionHeader header: JTAppleHeaderView, date: (startDate: NSDate, endDate: NSDate), identifier: String) {
        switch identifier {
        case "WhiteSectionHeaderView":
            let headerCell = (header as? WhiteSectionHeaderView)
            headerCell?.title.text = "Design multiple headers"
        default:
            let headerCell = (header as? PinkSectionHeaderView)
            headerCell?.title.text = "In any color or size you want"
        }
    }
    
}
