//
//  CellControl.swift
//  CalendarTest
//
//  Created by Don Rathwell May 26th 2016
//  Copyright © 2016 Don Rathwell. All rights reserved.
//
//

import Foundation
import UIKit
import JTAppleCalendar


import EventKit




class CellViewer: JTAppleDayCellView {
    
    
    //Colors for cells
    @IBInspectable var todayColor: UIColor!// = UIColor(red: 254.0/255.0, green: 73.0/255.0, blue: 64.0/255.0, alpha: 0.3)
    @IBInspectable var normalDayColor: UIColor! //UIColor(white: 0.0, alpha: 0.1)
    //var normalDayColor = UIColor.blackColor()
    var weekendDayColor = UIColor.grayColor()
    let textSelectedColor = UIColor.yellowColor()
    let textDeselectedColor = UIColor.blackColor()
    let previousMonthTextColor = UIColor.grayColor()
    
    //outlets for CellViewer
   
    @IBOutlet weak var selectedView: AnimationView!
    @IBOutlet weak var dateLabel: UILabel!
    @IBOutlet weak var noteImage: UIImageView!
   
    // Wolf Head Image stuff
    var badWolf = UIImage()
    var goodWolf = UIImage()
    let wolfHeadSize = CGSizeMake(30,30)
    var titles = [String]()
    var successCount = 0
    var failureCount = 0
    
   //Uncertain about this stuff
    lazy var todayDate : String = {
        [weak self] in
        let aString = self!.c.stringFromDate(NSDate())
        return aString
        }()
    lazy var c : NSDateFormatter = {
        let f = NSDateFormatter()
        f.dateFormat = "yyyy-MM-dd"
        
        return f
    }()
    
    //Array for event info
    var eventArray : [EKEvent] = []
    var tempCal: NSCalendar! = NSCalendar(calendarIdentifier: NSCalendarIdentifierGregorian)

    
    //Call other functions to customize cell
    func setupCellBeforeDisplay(cellState: CellState, date: NSDate) {
        
        noteImage.hidden = true
        //Set get images and set sizes
        setWolfImages()
        
        // Setup Cell text
        dateLabel.text =  cellState.text
       
        
        // Setup text color
        configureTextColor(cellState)
        
        //Setup display of notes
        configureDotColor(cellState)
        
        // Setup Cell Background color
        self.backgroundColor = c.stringFromDate(date) == todayDate ? todayColor:normalDayColor
        
        // Setup cell selection status
        configueViewIntoBubbleView(cellState)
        
        // Configure Visibility
        configureVisibility(cellState)
        
        //Configure events
        configureEvents(cellState)
    }
    
    //match event to calendar
    func configureEvents(cellState: CellState){
        getEventInfo()
        
        //figure out which image
       for event in eventArray {
            if tempCal.isDate(event.startDate, inSameDayAsDate : cellState.date) {
                if event.title == "success"{
                    successCount += 1
                }else if event.title == "failure"{
                    failureCount += 1
            }
        }
    }
        
        
        // put image in
        for event in eventArray {
            if tempCal.isDate(event.startDate, inSameDayAsDate : cellState.date) {
                if failureCount > successCount {
                    noteImage.image = badWolf
                } else {
                    noteImage.image = goodWolf
                }
                noteImage.hidden = false
                }
            }
        }
    
    func setWolfImages(){
        
        if let tempImage = UIImage(named:"badWolf2"){
            badWolf = imageResize(tempImage, sizeChange: wolfHeadSize)
        }
        if let tempImage2 = UIImage(named:"goodWolf3") {
            goodWolf = imageResize(tempImage2, sizeChange: wolfHeadSize)
        }
         noteImage.contentMode = UIViewContentMode.ScaleAspectFill
    }
    
    func imageResize (image: UIImage, sizeChange:CGSize)-> UIImage{
        
        let hasAlpha = true
        let scale: CGFloat = 0.0 // Use scale factor of main screen
        
        UIGraphicsBeginImageContextWithOptions(sizeChange, !hasAlpha, scale)
        image.drawInRect(CGRect(origin: CGPointZero, size: sizeChange))
        
        let scaledImage = UIGraphicsGetImageFromCurrentImageContext()
        return scaledImage
    }
    
    //to get event information
    func getEventInfo(){
        
        let eventStore = EKEventStore()
        let calendars = eventStore.calendarsForEntityType(.Event)
        
        for calendar in calendars {
            if calendar.title == "HabitName" {
                let oneMonthAgo = NSDate(timeIntervalSinceNow: -30*24*3600)
                let oneMonthAfter = NSDate(timeIntervalSinceNow: +30*24*3600)
                let predicate = eventStore.predicateForEventsWithStartDate(oneMonthAgo, endDate: oneMonthAfter, calendars: [calendar])
                let events = eventStore.eventsMatchingPredicate(predicate)
                for event in events {
                    eventArray.append(event)
                }
            }
        }
    }
    
    func configureVisibility(cellState: CellState) {
        if
            cellState.dateBelongsTo == .ThisMonth ||
                cellState.dateBelongsTo == .PreviousMonthWithinBoundary ||
                cellState.dateBelongsTo == .FollowingMonthWithinBoundary {
            self.hidden = false
        } else {
            self.hidden = false
        }
    }
    
    
    func configureDotColor(cellState: CellState) {
        
        let chek = Int(cellState.text)
        
        
        if chek! % 2 == 0 {
            //noteLabel.backgroundColor = UIColor.yellowColor()
        } else if cellState.dateBelongsTo == .ThisMonth {
            dateLabel.textColor = textDeselectedColor
        } else {
            dateLabel.textColor = previousMonthTextColor
        }
    }
    
    func configureTextColor(cellState: CellState) {
        if cellState.isSelected {
            dateLabel.textColor = textSelectedColor
        } else if cellState.dateBelongsTo == .ThisMonth {
            dateLabel.textColor = textDeselectedColor
        } else {
            dateLabel.textColor = previousMonthTextColor
        }
    }
    
    func cellSelectionChanged(cellState: CellState) {
        if cellState.isSelected == true {
            
            if selectedView.hidden == true {
                //configueViewIntoBubbleView(cellState)
                self.userInteractionEnabled = false
                selectedView.animateWithBounceEffect(withCompletionHandler: {
                    self.userInteractionEnabled = true
                })
            }
        } else {
            //configueViewIntoBubbleView(cellState, animateDeselection: true) ************************* THIS IS STILL A PROBLEM
        }
        
    }
    
    private func configueViewIntoBubbleView(cellState: CellState, animateDeselection: Bool = false) {
        if cellState.isSelected {
            
            delayRunOnMainThread(0.0, closure: {
                self.selectedView.layer.cornerRadius =  self.selectedView.frame.width  / 2
                self.selectedView.hidden = false
            })
            
            configureTextColor(cellState)
            
        } else {
            if animateDeselection {
                configureTextColor(cellState)
                if selectedView.hidden == false {
                    self.userInteractionEnabled = false
                    selectedView.animateWithFadeEffect(withCompletionHandler: { () -> Void in
                        self.userInteractionEnabled = true
                        self.selectedView.hidden = true
                        self.selectedView.alpha = 1
                    })
                }
            } else {
                //selectedView.hidden = true
            }
        }
    }
    
    func delayRunOnMainThread(delay:Double, closure:()->()) {
        dispatch_after(
            dispatch_time(
                DISPATCH_TIME_NOW,
                Int64(delay * Double(NSEC_PER_SEC))
            ),
            dispatch_get_main_queue(), closure)
    }

}
extension UIColor {
    convenience init(colorWithHexValue value: Int, alpha:CGFloat = 1.0){
        self.init(
            red: CGFloat((value & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((value & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(value & 0x0000FF) / 255.0,
            alpha: alpha
        )
    }
}
class AnimationView: UIView {
    
    func animateWithFlipEffect(withCompletionHandler completionHandler:(()->Void)?) {
        AnimationClass.flipAnimation(self, completion: completionHandler)
    }
    func animateWithBounceEffect(withCompletionHandler completionHandler:(()->Void)?) {
        let viewAnimation = AnimationClass.BounceEffect()
        viewAnimation(self){ _ in
            completionHandler?()
        }
    }
    func animateWithFadeEffect(withCompletionHandler completionHandler:(()->Void)?) {
        let viewAnimation = AnimationClass.FadeOutEffect()
        viewAnimation(self) { _ in
            completionHandler?()
        }
    }
}