//
//  ViewController.swift
//  CalendarTest
//
//  Created by Don Rathwell May 26th 2016
//  Copyright © 2016 Don Rathwell. All rights reserved.
//

import UIKit
import EventKit
import JTAppleCalendar

class ViewController: UIViewController {

    var savedEventId : String = ""
    var globalTimeStamp: String = ""
    var globalCalendar: EKCalendar?
    
    
    @IBOutlet weak var noteInputField: UITextField!
    
    var placeHolder : String = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        //Check for calendar and create if it doesn't exist
        calendarSetUp()
        assignbackground()
       
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func assignbackground(){
        let background = UIImage(named: "background1.jpg")
        var imageView : UIImageView!
        imageView = UIImageView(frame: view.bounds)
        imageView.contentMode =  UIViewContentMode.ScaleAspectFit
        imageView.clipsToBounds = true
        imageView.image = background
        imageView.center = view.center
        view.addSubview(imageView)
        self.view.sendSubviewToBack(imageView)
    }
    

    
    func calendarSetUp()->Void{
        let eventStore = EKEventStore()
        let calendars = eventStore.calendarsForEntityType(EKEntityType.Event) as [EKCalendar]
        var exists = false
        for calendar in calendars {
            if calendar.title == "HabitName" {
                self.globalCalendar = calendar
                exists = true
            }
        }
        if exists==false {
            let newCalendar = EKCalendar(forEntityType:EKEntityType.Event, eventStore:eventStore)
            newCalendar.title="HabitName"
            newCalendar.source = eventStore.defaultCalendarForNewEvents.source //Not sure about this
            self.globalCalendar = newCalendar
            do
            {
            try eventStore.saveCalendar(newCalendar, commit:true)
            
            }catch{
            print("Goofiness Abounds")
            }
        }
    }
    
    // Function to get UID of this habits calendar
    func getPSACalendar() -> EKCalendar{
        var calUID:String = "?"
        let eventStore = EKEventStore()
        let calendars = eventStore.calendarsForEntityType(EKEntityType.Event) as [EKCalendar]
        for calendar in calendars {
            if calendar.title == "HabitName" {
                calUID = calendar.calendarIdentifier
            }
        }
        let retCal = eventStore.calendarWithIdentifier(calUID)
            return retCal!
    }

    
    // Creates an event in the EKEventStore. The method assumes the eventStore is created and 
    // accessible
    func createEvent(eventStore: EKEventStore, title: String, startDate: NSDate, endDate: NSDate) {
        let event = EKEvent(eventStore: eventStore)
        //print("debug")
        event.title = title
        event.startDate = startDate
        event.endDate = endDate
        event.calendar = getPSACalendar()
        event.notes = noteInputField.text
        self.placeHolder = noteInputField.text!
        do {
            try eventStore.saveEvent(event, span: .ThisEvent)
            savedEventId = event.eventIdentifier
        } catch {
            print("Bad things happened")
        }
        
        /* StackOverfow for getting rid of that darn no shared claendars error.
 func createEvent(){
 var event = EKEvent(eventStore: self.eventStore)
 var calendar : EKCalendar!
 let calendars : [EKCalendar] = self.eventStore.calendarsForEntityType(EKEntityTypeEvent) as [EKCalendar]
 
 for aCal in calendars
 {
 if(aCal.calendarIdentifier == self.calendarIdentifier)
 {
 calendar = aCal
 break
 }
 } ...continue to do stuff to events....
 
 }*/
 
    }
 
    // Removes an event from the EKEventStore. The method assumes the eventStore is created and
    // accessible
    func deleteEvent(eventStore: EKEventStore, eventIdentifier: String) {
        let eventToRemove = eventStore.eventWithIdentifier(eventIdentifier)
        if (eventToRemove != nil) {
            do {
                try eventStore.removeEvent(eventToRemove!, span: .ThisEvent)
            } catch {
                print("Bad things happened")
            }
        }
    }
    
    //Funcion to get current date/time
    func getTimeStamp() {
        let timestamp = NSDateFormatter.localizedStringFromDate(NSDate(), dateStyle: .MediumStyle, timeStyle: .ShortStyle)
        self.globalTimeStamp = timestamp
        print(timestamp)
    }
    
    // Responds to button to add event. This checks that we have permission first, before adding the
    // event
    @IBAction func successEvent(sender: UIButton) {
        let eventStore = EKEventStore()
        
        let startDate = NSDate()
        let endDate = startDate.dateByAddingTimeInterval(1) // One second
        getTimeStamp()
        if (EKEventStore.authorizationStatusForEntityType(.Event) != EKAuthorizationStatus.Authorized) {
            eventStore.requestAccessToEntityType(.Event, completion: {
                granted, error in
                self.createEvent(eventStore, title: String(startDate), startDate: startDate, endDate: endDate)
            })
        } else {
            createEvent(eventStore, title: "success", startDate: startDate, endDate: endDate)
        }
    }


    // Responds to button to failure event.
    @IBAction func failureEvent(sender: UIButton) {
        let eventStore = EKEventStore()
        let startDate = NSDate()
        let endDate = startDate.dateByAddingTimeInterval(1) // One second
        getTimeStamp()
        if (EKEventStore.authorizationStatusForEntityType(.Event) != EKAuthorizationStatus.Authorized) {
            eventStore.requestAccessToEntityType(.Event, completion: {
                granted, error in
                self.createEvent(eventStore, title: String(startDate), startDate: startDate, endDate: endDate)
            })
        } else {
            createEvent(eventStore, title: "failure", startDate: startDate, endDate: endDate)
        }
    }
}

